/*
 * Copyright(c) 2010, Norio Kobota, All rights reserved.
 */

#include <string.h>

#include "mod_websocket.h"

static int send_forward(handler_ctx *hctx, char *payload, size_t siz) {
    const char endl = '\0';
    buffer *b = NULL;

    if (payload == NULL || siz == 0) {
        return 0;
    }
    b = chunkqueue_get_append_buffer(hctx->tocli);
    if (!b) {
        DEBUG_LOG(MOD_WEBSOCKET_LOG_ERR, "s", "no memory");
        return -1;
    }
    buffer_append_memory(b, payload, siz);
    /* needs '\0' char to send */
    buffer_append_memory(b, &endl, 1);
    return 0;
}

static int recv_forward(handler_ctx *hctx) {
    chunk *c = NULL;
    buffer *frame = NULL, *b = NULL;

    DEBUG_LOG(MOD_WEBSOCKET_LOG_DEBUG, "sdsx",
              "recv data from client ( fd =", hctx->con->fd,
              "), size =", chunkqueue_length(hctx->fromcli));
    for (c = hctx->fromcli->first; c; c = c->next) {
        frame = c->mem;
        if (!frame) {
            continue;
        }
        b = chunkqueue_get_append_buffer(hctx->tosrv);
        if (!b) {
            DEBUG_LOG(MOD_WEBSOCKET_LOG_ERR, "s", "no memory");
            return -1;
        }
        buffer_append_memory(b, frame->ptr, frame->used);
    }
    chunkqueue_reset(hctx->fromcli);
    return 0;
}

int mod_websocket_frame_send(handler_ctx *hctx, mod_websocket_frame_type_t type,
                             char *payload, size_t siz) {
    if (!hctx) {
        return -1;
    }
    if (hctx->mode == MOD_WEBSOCKET_WEBSOCKET_PROXY) {
        return send_forward(hctx, payload, siz);
    } else if (hctx->mode == MOD_WEBSOCKET_TCP_PROXY) {
        DEBUG_LOG(MOD_WEBSOCKET_LOG_ERR, "s", "invalid mode TCP PROXY");
    }
    return -1;
}

int mod_websocket_frame_recv(handler_ctx *hctx) {
    if (!hctx) {
        return -1;
    }
    if (hctx->mode == MOD_WEBSOCKET_WEBSOCKET_PROXY) {
        return recv_forward(hctx);
    } else if (hctx->mode == MOD_WEBSOCKET_TCP_PROXY) {
        DEBUG_LOG(MOD_WEBSOCKET_LOG_ERR, "s", "invalid mode TCP PROXY");
    }
    return -1;
}
